/*

適用晶片 SIMCOM SIM7500SA

*/

const Log=require('./log');

module.exports = function(obj){
  //var sprintf = require('sprintf').sprintf;
  var sprintf = require("sprintf-js").sprintf;
  var pdu = require('pdu');
  var moment = require('moment');
  var UART=require('./uart');
  
  var _this=this;
  _this.obj=obj;
  _this.uart=null;
  _this.test=function(data){
    _this.uart.onData(data);
  };
  _this.lte_reset=function(){
    console.log('[lte_reset]');
    var sendMessage=_this.sendMessage;
    sendMessage('AT+CRESET');
  };
  
  _this.bufToStr=function(buf){
    var s="<<<Buffer>>>\n";
    for(i=0;i<buf.length;i++){
      s+= Buffer.from([buf[i]]).toString('hex') +" ";
      if(i % 16==15)
        s+="\n";
    }  
    return s;
  }

  _this.at_command_list=[];
  
  setInterval(function(){
    if(_this.at_command_list.length){
      cmd=_this.at_command_list.shift();
      //console.log('at_command_list',_this.at_command_list.length);
      if(_this.uart!=null)
        _this.uart.write(cmd);
      
      //if(cmd=="AT+CRESET"){
      //  setTimeout(function(){
      //    _this.reboot(); 
      //  },500);
      //}
    }
  },300);
  
  _this.sendMessage=function(cmd,first){
    if(first==true){
      _this.at_command_list.unshift(cmd);
    }
    else
      _this.at_command_list.push(cmd);
  };
  
  _this.reboot=function(){  //重開機
    var exec = require('child_process').exec;
    
    Log.send('CMD','[REBOOT]');
    exec('reboot', function(error, stdout, stderr){ 
    });
  };
  _this.shutdown=function(){  //shutdown
    var exec = require('child_process').exec;
    
    Log.send('CMD','[shutdown]');
    exec('shutdown -h 0', function(error, stdout, stderr){ 
      console.log('[shutdown]',error, stdout, stderr);
    });
  };
  
  _this.reset=function (){
    var sendMessage=_this.sendMessage;
    sendMessage('AT+CNVW=1016,0,"01"');
    sendMessage('AT+CNVW=1017,0,"02"');
    sendMessage('AT+CNMI=2,1,2,2,1');
    sendMessage('AT+CSCB=0,"911,919,4370-4380,4383-4393,12345",""');
    sendMessage('AT+CMGF=0');
    sendMessage('AT+CGPSAUTO=1');
    sendMessage('AT+CRESET');
  };  
  
  
  _this.reTest=function (idx){
    var sendMessage=_this.sendMessage;
    sendMessage('AT+CMGF=0');
    sendMessage('AT+CPMS="SM","SM","SM"');
    sendMessage('AT+CPSI?');
    sendMessage('AT+CSCB=0,"911,919,4370-4380,4383-4393,12345",""');
    
    if(idx==0)
      sendMessage('AT+CGPSINFO');
    
    if(idx>=5*20)
      idx=-1;
    setTimeout(function(){ _this.reTest(idx+1); },3000);
  }
  _this.onJSON=function(json){
    //reset
    if(json.CMD=='reset'){
      Log.send('CMD','[RESET]');
      _this.reset();
    }
    else if(json.CMD=='wifi'){
      var wifi_conf="ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev\n"+
                    "update_config=1\n"+
                    "country=TW\n"+
                    "\n"+
                    "network={\n"+
                    "	ssid=\""+json.data.name+"\"\n"+
                    "	psk=\""+json.data.pswd+"\"\n"+
                    "}";

      Log.send('text','[WIFI] '+wifi_conf);

      var fs = require('fs');
      fs.writeFile("/data/wifi.conf", wifi_conf, function(err) {
        if(!err) {
          setTimeout(function(){ _this.reboot(); } ,500);
        }
      }); 
    }
  };
  _this.controller=[
    '0966603808',
    '0903572061',
  ];
  _this.onSMS=function(_sms){
    var text=_sms.text;
    text=text.replace(/\u0000/g, '');
    text=text.replace(/\u001b\(/g, '{');
    text=text.replace(/\u001b\)/g, '}');
    text=decodeURIComponent(text);
    
    
    code=false;
    if(_this.controller.indexOf(_sms.sender)!==-1){
      if(text.indexOf("JSON:")==0){  //JSON
        json=text.substring(5);
        try{
          data=JSON.parse(json);
          _this.onJSON(data);
          code=true;
        } catch(e) {
        }
      }
    }
    
    if(!code)
      _this.obj.onSMS(text,moment(_sms.time).format('YYYY-MM-DDTHH:mm:ss.SSSZZ').toString(),_sms.sender);    
  }
  _this.sms_buffer={};
  _this.sms_key=false;
  
  
  _this.cbs_item={};
  //_this.cbs_buffer={};
  //_this.cbs_key=false;
  
  _this.mode="--";
  _this.reTest_inited=false;
  function init(){
    _this.uart=new UART({
      'connect':{
        'portId': '/dev/ttyS0',
        'baudRate': 115200,
        'dataBits': 8,
        'stopBits': 1,
        'parity': 'none',
      },
      'debug':_this.obj.debug,
      'onInited':function(){
        if(_this.obj.debug)
          console.log('[uart] connecting');
        if(_this.uart!=null)
          _this.uart.connect();
      },
      'onOpened':function(){
        if(_this.obj.debug)
          console.log('[uart] onOpened');
        
        if(!(process.argv[2] & 2)){
          if(!_this.reTest_inited){
            _this.reTest_inited=true;
            _this.reTest(0);
          }
        }
          
      },    
      'onErrorClose': function(e){
        console.log('[onErrorClose]',e);
        if((typeof(_this.uart)!=='undefined') && (_this.uart!=null)){
          console.log('[delete] delete uart');
          delete _this.uart;
          _this.uart=null;
        }
        setTimeout(function(){
          console.log('[reconnect]',Math.random());
          init();
        },1000);
      },
      'onData':function(data){
        if(_this.obj.debug){
          console.log('onData',_this.bufToStr(data));
        }
      },
      'onReadLine':function(str){
        if(str.trim().length>0){
          if(_this.obj.debug)
            console.log('onReadLine','[msg] '+str);
          switch(_this.mode){
            case "SMSPDU":
              {
                _this.mode="--";
                var sms=pdu.parse(str);
                
                if(_this.obj.debug)
                  Log.send('SMSPDU',sms.text);
                
                if(typeof(sms.udh)=== 'undefined'){
                  _sms={
                    'text': sms.text,
                    'sender': sms.sender,
                    'time': sms.time,
                  };
                  _this.onSMS(_sms);
                }
                else{
                  _this.sms_key=sms.udh.reference_number;
                  if(typeof(_this.sms_buffer[_this.sms_key])=== 'undefined'){
                    _this.sms_buffer[_this.sms_key]={
                      'text': "",
                      'sender': sms.sender,
                      'time': sms.time,
                    };
                  }
                  _this.sms_buffer[_this.sms_key].text+=sms.text;
                  _this.sms_buffer[_this.sms_key].time=sms.time;
                  if(sms.udh.current_part==sms.udh.parts){ //表示所有簡訊都接收完畢
                    _this.onSMS(_this.sms_buffer[_this.sms_key]);
                    delete _this.sms_buffer[_this.sms_key];
                  }
                }
              }
              break;
            case "CBS":
              {
                _this.mode="--";
                _this.cbs_item.data+=str;
                if(_this.cbs_item.page==_this.cbs_item.pages){
                  
                  var data=_this.cbs_item.data;
                  //UCS2 to UTF8!?, UCS2好像是UTF-16的子集合!?
                  var text = '';
                  for(var i = 0;i<=data.length;i=i+4)
                    text += String.fromCharCode("0x"+data[i]+data[i+1]+data[i+2]+data[i+3]);
                  _this.obj.onCBS(text,moment().format('YYYY-MM-DDTHH:mm:ss.SSSZZ').toString());
                }
              }
              break;
            case "CBSPDU":
              {
                _this.mode="--";
                var page=parseInt(str.substr(10,1),16);
                if(page==1){
                  _this.cbs_item={
                    'id':       parseInt(str.substr(0,4),16),  //((buf[0]<<8)+buf[1]),
                    'channel':  parseInt(str.substr(4,4),16),  //((buf[2]<<8)+buf[3]),
                    'encode':   parseInt(str.substr(8,2),16),  //buf[4],
                    'page':     page,
                    'pages':    parseInt(str.substr(11,1),16),
                    'data':     "",
                  };
                  //Log.send('text','[CBSPDU] '+sms.text + '<Br />'+JSON.stringify(_this.cbs_item));
                }
                else{
                  _this.cbs_item.page= page;
                  _this.cbs_item.pages= parseInt(str.substr(11,1),16);
                }
                
                var txt=str.substr(12);
                _this.cbs_item.data+=txt;
                
                if(_this.cbs_item.page==_this.cbs_item.pages){
                  var data=_this.cbs_item.data;
                  //UCS2 to UTF8!?, UCS2好像是UTF-16的子集合!?
                  var text = '';
                  for(var i = 0;i<=data.length;i=i+4)
                    text += String.fromCharCode("0x"+data[i]+data[i+1]+data[i+2]+data[i+3]);                  
                  _this.obj.onCBS(text,moment().format('YYYY-MM-DDTHH:mm:ss.SSSZZ').toString());
                }
              }
              break;
            default:
              if((data=str.match(/(\+CMTI: \"SM\",)(\d+)*/i))!=null){  //收到簡訊
                _this.sendMessage("AT+CMGRD="+parseInt(data[2]),1); //打開簡訊
              }
              else if((data=str.match(/(\+CMGRD: )(\d+),\"\",(\d+)/i))!=null){  //簡訊 PDU Mode
                _this.mode="SMSPDU";
              }
              //+CSCB: 0,"911,919,12345,4370-4380,4383-4393",""
              if((data=str.match(/(\+CSCB: 0,")([0-9a-z\-,]+)(","")/i))!=null){  //頻道設定值
              
              }
              //+CPSI: NO SERVICE,Online
              //+CPSI: LTE,Online,466-97,0x2CC4,59144305,11,EUTRAN-BAND28,9560,5,5,-176,-923,-548,7
              else if((data=str.match(/(\+CPSI: )/i))!=null){  //4G訊號
                var data=str.split(",");
                _this.obj.onStatus(
                  (data[0]=='+CPSI: LTE' && data[1]=='Online'),
                  moment().format('YYYY-MM-DDTHH:mm:ss.SSSZZ').toString());
              }
              //+CGPSINFO: ,,,,,,,,
              //+CGPSINFO: 2458.966726,N,12132.483547,E,090318,064957.0,5.0,0.0,199.4
              else if((data=str.match(/(\+CGPSINFO: )/i))!=null){  //GPS
                var data=str.split(/,| /);
                var coordValue=function(s){
                  if(s.length>0){
                    var v=parseFloat(s)/100;
                    return Math.floor(v)+(v-Math.floor(v))*100/60;
                  }
                  return 0;
                }
                var item={
                  'lat'     :  coordValue(data[1])+data[2],
                  'lng'     :  coordValue(data[3])+data[4],
                  'time'    :  0,
                  'alt'     :  parseFloat(data[7]),
                  'speed'   :  parseFloat(data[8]),
                  'course'  :  parseFloat(data[9]),
                };
                
                if(data[5].length>0){
                  item.time=moment(data[5]+data[6]+" +0000",'DDMMYYHHmmss.S Z').format('YYYY-MM-DDTHH:mm:ss.SSSZZ').toString();
                  if(item.lat!=0 && item.lng!=0)
                    Log.send('GLoc',JSON.stringify(item));
                }
              }
              else if((data=str.match(/(\+CBM: )(\d+),(\d+),(\d+),(\d+),(\d+)/i))!=null){ //CBS-TEXT訊息
                _this.mode="CBS";
                var page=parseInt(data[5]);
                if(page==1){
                  _this.cbs_item={
                    'id': parseInt(data[2]),
                    'channel': parseInt(data[3]),
                    'encode': parseInt(data[4]),
                    'page': page,
                    'pages': parseInt(data[6]),
                    'data': "",
                  };
                }
                else{
                  _this.cbs_item.page= page;
                  _this.cbs_item.pages= parseInt(data[6]);
                }
              }
              else if((data=str.match(/(\+CBM: )(\d+)/i))!=null){ //CBS-PDU訊息
                _this.mode="CBSPDU";
              }
              else{
              }
              break;
          };
        }
      },
    });
  }
  init();
};